<?php

namespace common\logic\Order\Order;

use common\helpers\GlobalParam;
use common\helpers\Price;
use common\logic\BaseService;
use common\logic\Document\Document\Document;
use common\logic\Document\Document\DocumentSolver;
use common\logic\Order\ProductOrder\ProductOrder;
use common\logic\Producer\Producer\Producer;
use common\logic\Product\Product\Product;
use common\logic\SolverInterface;
use yii\helpers\Html;

class OrderSolver extends BaseService implements SolverInterface
{
    protected DocumentSolver $documentSolver;

    public function __construct()
    {
        $this->documentSolver = $this->loadService(DocumentSolver::class);
    }

    public function getFieldNameAmount($typeTotal = Order::AMOUNT_TOTAL, $typeField = ''): string
    {
        $fieldName = 'amount';
        if ($typeTotal == Order::INVOICE_AMOUNT_TOTAL) {
            $fieldName = 'invoice_amount';
        }

        if ($typeField == 'vat') {
            $fieldName = $fieldName . '_vat';
        } elseif ($typeField == 'with_tax') {
            $fieldName = $fieldName . '_with_tax';
        }

        return $fieldName;
    }

    public function getTotalVat(Order $order, $typeTotal = Order::AMOUNT_TOTAL): float
    {
        $fieldName = $this->getFieldNameAmount($typeTotal, 'vat');
        $totalVat = 0;

        foreach ($order->$fieldName as $vat) {
            $totalVat += $vat;
        }

        return $totalVat;
    }

    /**
     * Retourne le statut de paiement de la commande (payée, surplus, ou impayée).
     */
    public function getPaymentStatus(Order $order): ?string
    {
        // payé
        if ($this->getAmountWithtax($order) - $this->getAmount($order, Order::AMOUNT_PAID) < 0.01 &&
            $this->getAmountWithtax($order) - $this->getAmount($order, Order::AMOUNT_PAID) > -0.01) {
            return Order::PAYMENT_PAID;
        } // à rembourser
        elseif ($this->getAmountWithtax($order) - $this->getAmount($order, Order::AMOUNT_PAID) <= -0.01) {
            return Order::PAYMENT_SURPLUS;
        } // reste à payer
        elseif ($this->getAmountWithtax($order) - $this->getAmount($order, Order::AMOUNT_PAID) >= 0.01) {
            return Order::PAYMENT_UNPAID;
        }

        return null;
    }

    /**
     * Retourne le résumé du point de vente lié à la commande au format HTML.
     */
    public function getPointSaleSummary(Order $order): string
    {
        $html = '';

        if (isset($order->pointSale)) {
            $html .= '<span class="name-point-sale">' .
                Html::encode($order->pointSale->name) .
                '</span>' .
                '<br /><span class="locality">'
                . Html::encode($order->pointSale->locality)
                . '</span>';

            if (strlen($order->comment_point_sale)) {
                $html .= '<div class="comment"><span>'
                    . Html::encode($order->comment_point_sale)
                    . '</span></div>';
            }
        } else {
            $html .= 'Point de vente supprimé';
        }

        return $html;
    }

    /**
     * Retourne une chaine de caractère décrivant l'utilisateur lié à la commande.
     */
    public function getStrUser(Order $order): string
    {
        if (isset($order->user)) {
            if (isset($order->user->name_legal_person) && strlen($order->user->name_legal_person)) {
                return Html::encode($order->user->name_legal_person);
            } else {
                $strUser = $order->user->lastname;
                if ($order->user->lastname && $order->user->name) {
                    $strUser .= ' ' . $order->user->name;
                }
                return Html::encode($strUser);
            }
        } elseif (strlen($order->username)) {
            return Html::encode($order->username);
        } else {
            return 'Client introuvable';
        }
    }

    public function isLinkedToValidDocument(Order $order): bool
    {
        return ($order->deliveryNote && $this->documentSolver->isStatusValid($order->deliveryNote))
            || ($order->quotation && $this->documentSolver->isStatusValid($order->quotation))
            || ($order->invoice && $this->documentSolver->isStatusValid($order->invoice));
    }

    public function getCommentReport(Order $order): string
    {
        $comment = '';
        $hasComment = false;

        if ($order->comment && strlen($order->comment) > 0) {
            $hasComment = true;
            $comment .= $order->comment;
        }

        if ($order->delivery_home && $order->delivery_address && strlen($order->delivery_address) > 0) {
            if ($hasComment) {
                $comment .= '<br /><br />';
            }

            $comment .= '<strong>Livraison à domicile :</strong><br />';
            $comment .= nl2br($order->delivery_address);
        }

        return $comment;
    }

    public function getUsername(Order $order): string
    {
        $username = '';
        if ($order->user) {
            $username = $order->user->getUsername();
        }
        if (strlen($order->username)) {
            $username = $order->username;
        }
        return $username;
    }

    /**
     * Retourne un bloc html présentant une date.
     */
    public function getBlockDate(Order $order): string
    {
        return '<div class="block-date">
                    <div class="day">' . strftime('%A', strtotime($order->distribution->date)) . '</div>
                    <div class="num">' . date('d', strtotime($order->distribution->date)) . '</div>
                    <div class="month">' . strftime('%B', strtotime($order->distribution->date)) . '</div>
                </div>';
    }

    /**
     * Retourne le nombre de produits commandés.
     */
    public function countProducts(Order $order)
    {
        if ($order->productOrder && is_array($order->productOrder)) {
            return count($order->productOrder);
        }

        return 0;
    }

    public function getProductQuantityPieces(Product $product, array $orders): int
    {
        $quantity = 0;

        if (isset($orders) && is_array($orders) && count($orders)) {
            foreach ($orders as $c) {
                if (is_null($c->date_delete)) {
                    foreach ($c->productOrder as $po) {
                        if ($po->id_product == $product->id) {
                            if ($po->unit == 'piece') {
                                $quantity += $po->quantity;
                            } else {
                                if (isset($po->product) && $po->product->weight > 0) {
                                    $quantity += ($po->quantity * Product::$unitsArray[$po->unit]['coefficient']) / $po->product->weight;
                                }
                            }
                        }
                    }
                }
            }
        }

        return $quantity;
    }

    /**
     * Retourne la quantité d'un produit donné de plusieurs commandes.
     */
    public function getProductQuantity(Product $product, array $orders, $ignoreCancel = false, $unit = null): int
    {
        $quantity = 0;

        if (isset($orders) && is_array($orders) && count($orders)) {
            foreach ($orders as $c) {
                if (is_null($c->date_delete) || $ignoreCancel) {
                    foreach ($c->productOrder as $po) {
                        if ($po->id_product == $product->id &&
                            ((is_null($unit) && $po->product->unit == $po->unit) || (!is_null($unit) && strlen(
                                        $unit
                                    ) && $po->unit == $unit))) {
                            $quantity += $po->quantity;
                        }
                    }
                }
            }
        }

        return $quantity;
    }

    /**
     * Retourne une classe identifiant l'historique de la commande (ajoutée,
     * modifiée, supprimée).
     */
    public function getClassHistory(Order $order): string
    {
        if (!is_null($order->date_delete)) {
            return 'commande-delete';
        }

        if (!is_null($order->date_update)) {
            return 'commande-update';
        }

        return 'commande-create';
    }

    /**
     * Retourne l'historique de la commande (ajoutée, modifiée, supprimée) au format HTML.
     */
    public function getStrHistory(Order $order): string
    {
        $arr = [
            'class' => 'create',
            'glyphicon' => 'plus',
            'str' => 'Ajoutée',
            'date' => $order->date
        ];

        if (!is_null($order->date_update)) {
            $arr = [
                'class' => 'update',
                'glyphicon' => 'pencil',
                'str' => 'Modifiée',
                'date' => $order->date_update
            ];
        }
        if (!is_null($order->date_delete)) {
            $arr = [
                'class' => 'delete',
                'glyphicon' => 'remove',
                'str' => 'Annulée',
                'date' => $order->date_delete
            ];
        }

        $html = '<div class="small"><span class="' . $arr['class'] . '">'
            . '<span class="glyphicon glyphicon-' . $arr['glyphicon'] . '"></span> '
            . $arr['str'] . '</span> le <strong>'
            . date('d/m/Y à G\hi', strtotime($arr['date'])) . '</strong></div>';

        return $html;
    }

    /**
     * Retourne l'origine de la commande (client, automatique ou admin) sous forme texte ou HTML.
     */
    public function getStrOrigin(Order $order, $withLabel = false): string
    {
        $classLabel = '';
        $str = '';

        if ($order->origin == Order::ORIGIN_USER) {
            $classLabel = 'success';
            $str = 'Client';
        } elseif ($order->origin == Order::ORIGIN_AUTO) {
            $classLabel = 'default';
            $str = 'Auto';
        } elseif ($order->origin == Order::ORIGIN_ADMIN) {
            $classLabel = 'warning';
            $str = 'Vous';
        }

        if ($withLabel) {
            return '<span class="label label-' . $classLabel . '">'
                . $str . '</span>';
        } else {
            return $str;
        }
    }

    /**
     * Retourne le montant de la commande (total, payé, restant, ou en surplus).
     */
    public function getAmount(Order $order, string $type = Order::AMOUNT_TOTAL, $format = false)
    {
        $amount = $order->amount;
        if ($type == Order::INVOICE_AMOUNT_TOTAL && $order->invoice_amount) {
            $amount = $order->invoice_amount;
        }

        return $this->_getAmountGeneric($order, $type, $amount, $format);
    }

    public function getAmountWithTax(Order $order, string $type = Order::AMOUNT_TOTAL, $format = false)
    {
        $amount = $order->amount + $this->getTotalVat($order, $type);
        if ($type == Order::INVOICE_AMOUNT_TOTAL && $order->invoice_amount) {
            $amount = $order->invoice_amount + $this->getTotalVat($order, $type);
        }

        return $this->_getAmountGeneric($order, $type, $amount, $format);
    }

    protected function _getAmountGeneric(Order $order, string $type, float $amountOrder, bool $format)
    {
        switch ($type) {
            case Order::AMOUNT_TOTAL :
            case Order::INVOICE_AMOUNT_TOTAL :
                $amount = $amountOrder;
                break;
            case Order::AMOUNT_PAID :
                $amount = $order->paid_amount;
                break;
            case Order::AMOUNT_REMAINING :
                $amount = $this->getAmountWithTax($order, Order::AMOUNT_TOTAL)
                    - $this->getAmountWithTax($order,Order::AMOUNT_PAID);
                break;
            case Order::AMOUNT_SURPLUS :
                $amount = $this->getAmountWithTax($order,Order::AMOUNT_PAID)
                    - $this->getAmountWithTax($order,Order::AMOUNT_TOTAL);
                break;
        }

        if ($format) {
            return Price::format($amount);
        } else {
            return $amount;
        }
    }


}